from .proxy import Proxy
from .gui import Gui

import threading

def main():
    """Entrypoint"""
    gui = Gui()
    proxy = Proxy()

    thread = threading.Thread(target=proxy.proxy_up)
    thread.start()

    gui.frame()

    thread.join()

if __name__ == '__main__':
    main()


from abc import ABCMeta, abstractmethod

class Observer(metaclass=ABCMeta):

    @abstractmethod
    def update(self, message=''):
        """Update the observers

        Args:
            message (str, optional): A message to deliver to the observers.
                Defaults to an empty string.
        """


from .libs.observer import Observer
from .proxy import Proxy

from appJar import gui

import re

class Gui(Observer):

    def __init__(self):
        """Constructor"""
        Proxy().register(self)
        self.title = 'pysniff'
        self.app = gui(showIcon=False)
        self.log = ''

    def frame(self):
        """Setup the basic application frame."""
        self.app.setSize('1080x600')
        self.app.setFont(8)
        
        self.app.startScrollPane('Pane')
        self.app.addMessage(self.title, self.log)
        self.app.registerEvent(self.updateWidget)
        self.app.stopScrollPane()
        self.app.go()

    def updateWidget(self):
        """Update event handler

        Updates the text on the text widget.
        """
        if self.log is not '':
            self.app.setMessage(self.title, self.log)

    def update(self, text=''):
        """Implementation of the observable update method.

        Updates the log text with new sniffed data.

        Args:
            text (str, optional): New log data. Defaults to an empty string.
        """
        text = re.sub(r'\n', '', text)
        self.log += re.sub(
            r'(GET|POST|PUT|DELETE|HEAD|CONNECT|OPTIONS|TRACE|PATCH)', 
            r'\n\n\1',
            text
        )
        print('Message: {}'.format(text.encode('utf-8', errors='ignore')))




class Subject:
    observers = []

    def register(self, observer):
        """Registeres an observer

        Args:
            observer (obj): An observer object to register.
        """
        if observer not in self.observers:
            self.observers.append(observer)

    def deregister(self, observer):
        """Deregister an observer

        Args:
            observer (obj): An observer to deregister.
        """
        self.observers.remove(observer)

    def notify(self, message=''):
        """Notify observers.

        Args:
            message (str, optional): A message to be sent to the registered 
                observers. Value defaults to an empty string.
        """
        for observer in self.observers:
            observer.update(message)


from setuptools import setup, find_packages

setup(
    name = 'pysniff',
    version = '0.0.1',
    description = 'A simple netowrk sniffing application',
    author = 'zWaR',
    author_email = 'zwar@sharklasers.com',
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7'
    ],
    keywords = 'network sniffer monitoring proxy',
    packages = ['pysniff', 'pysniff/libs'],
    setup_requires = ['pytest-runner'],
    install_requires = [
        'appjar'
    ],
    url="https://github.com/zWaR/pysniff",
    project_urls={
        "Source Code": "https://github.com/zWaR/pysniff.git",
        "Bug Tracker": "https://github.com/zWaR/pysniff/issues",
        "Documentation": "https://github.com/zWaR/pysniff"
    },
    entry_points = {
        'console_scripts': [
            'pysniff=pysniff.main:main'
        ]
    }
)


import re
import socket
import string
import sys
import codecs

from .libs.subject import Subject

class Proxy(Subject):

    def __init__(self):
        """Constructor
        """
        self.port = 8080
        self.host = 'localhost'
        self.log_filename = 'pysniff.log'
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        codecs.register_error('dotescape', self._dotescape)
    
    def bind(self):
        """Bind to a port.

        Bind the socker listener to a port.
        """
        try:
            self.socket.bind((self.host, self.port))
        except Exception as exception:
            print('Binding {} to {} failed with {}'
                .format(self.host, self.port, exception))

    def listen(self):
        """Start listening (accepting connection) on the open socket.
        """
        self.socket.listen(10)

    def close(self):
        """Close the socket.
        """
        self.socket.close()

    def write_log(self, data):
        """Write the sniffed data to a log.

        Writes the data to a log file. It also converts the binary object to
        a printable string.

        Args:
            data (obj): A binary object of data which has to be stored to a
                log.
        """
        with open(self.log_filename, 'a', encoding='utf-8') as log:
            text = data.decode('utf-8', 'dotescape')
            text = re.sub(
                r'(GET|POST|PUT|DELETE|HEAD|CONNECT|OPTIONS|TRACE|PATCH)', 
                r'\n\n\1', 
                text
            )
            log.write(text)
        self.notify(text)

    def proxy_up(self):
        """Starts the proxy.
        """
        self.bind()
        self.listen()
        while True:
            conn, addr = self.socket.accept()
            print('Connected with {}:{}'.format(self.host, self.port))
            data = conn.recv(1000)
            # print(data)
            text = self.write_log(data)
        self.close()

    def _dotescape(self, err):
        """Decode error handler.

        Escape error handler for the decode method. It handles the unicode
        decode errors, which are thrown when decode does not know how to
        convert a specific character.

        Args:
            err (obj): Error object.
        """
        # print(err, dir(err), err.start, err.end, err.object[:err.start])
        byte = err.object[err.start:err.end]
        reply = '.'
        if len(byte) == 1:
            reply = u'\\x'+hex(ord(byte))[2:]
        else:
            for i in range(len(byte)):
                reply = '{}.'.format(reply)
        return (reply, err.end)
